@@ -17,6 +17,7 @@ import (
 type Consequences struct {
 	context *Context
 	actions *Actions
+	timeout int
 }
 
 func (c *Consequences) Expect(e Expectation) *Consequences {
@@ -24,7 +25,7 @@ func (c *Consequences) Expect(e Expectation) *Consequences {
 	c.context.t.Helper()
 	var message string
 	var state state
-	timeout := time.Duration(15) * time.Second
+	timeout := time.Duration(c.timeout) * time.Second
 	for start := time.Now(); time.Since(start) < timeout; time.Sleep(3 * time.Second) {
 		state, message = e(c)
 		switch state {
@@ -84,3 +85,8 @@ func (c *Consequences) resource(kind, name, namespace string) ResourceStatus {
 		},
 	}
 }
+
+func (c *Consequences) Timeout(timeout int) *Consequences {
+	c.timeout = timeout
+	return c
+}
